@@ -1,4 +1,4 @@
-## Copyright 2009-2012 Luc Saffre
+## Copyright 2009-2013 Luc Saffre
 ## This file is part of the Lino project.
 ## Lino is free software; you can redistribute it and/or modify 
 ## it under the terms of the GNU General Public License as published by
@@ -32,9 +32,9 @@ from django.contrib.contenttypes.models import ContentType
 
 from lino.utils.instantiator import make_converter
 #~ from lino import reports
-from lino.core.coretools import get_data_elem # , get_unbound_meth
-import lino
-from lino.core import fields
+#~ from lino.core.fields import get_data_elem # , get_unbound_meth
+#~ import lino
+#~ from lino.core import fields
 from lino.utils import get_class_attr
 from lino.ui import requests as ext_requests
 
@@ -52,10 +52,14 @@ class ChoicesChooser(FieldChooser):
   
 
 class Chooser(FieldChooser):
+    """
+    A Chooser holds information about the possible choices of a field.
+    """
     #~ stored_name = None
     simple_values = False
     instance_values = True
     force_selection = True
+    #~ quick_search_fields = None
     #~ force_selection = True
     #~ on_quick_insert = None
     #~ quick_insert_field = None
@@ -71,6 +75,7 @@ class Chooser(FieldChooser):
             self.force_selection = getattr(meth,'force_selection',self.force_selection)
         #~ self.context_params = meth.func_code.co_varnames[1:meth.func_code.co_argcount]
         self.context_params = meth.context_params
+        #~ self.multiple = meth.multiple
         #~ self.context_params = meth.func_code.co_varnames[:meth.func_code.co_argcount]
         #~ print '20100724', meth, self.context_params
         #~ logger.warning("20100527 %s %s",self.context_params,meth)
@@ -96,7 +101,8 @@ class Chooser(FieldChooser):
         #~ m = get_unbound_meth(model,field.name + "_display")
         #~ if m is not None:
             #~ self.display_meth
-            
+        
+        #~ if get_class_attr(model,"create_%s_choice" % field.name):
         if hasattr(model,"create_%s_choice" % field.name):
             self.can_create_choice = True
             
@@ -115,12 +121,12 @@ class Chooser(FieldChooser):
                 return vf
         return self.model._meta.get_field(name)
             
-    def get_choices(self,**context):
+        
+    def get_choices(self,**context): # 20120918b
         "Return a list of choices for this chooser, using keyword parameters as context."
         args = []
         for varname in self.context_params:
             args.append(context.get(varname,None))
-        #~ print args
         return self.meth(*args)
       
     #~ def get_instance_choices(self,obj):
@@ -129,8 +135,13 @@ class Chooser(FieldChooser):
         #~ for varname in self.context_params:
             #~ args.append(getattr(obj,varname,None))
         #~ return self.meth(*args)
-      
+
+    #~ def get_request_choices(self,ar,tbl):
     def get_request_choices(self,request,tbl):
+        """
+        Return a list of choices for this chooser, 
+        using a HttpRequest to build the context.
+        """
         kw = {}
         
         # 20120202
@@ -156,10 +167,22 @@ class Chooser(FieldChooser):
             kw[str(k)] = v
         for cv in self.converters:
             kw = cv.convert(**kw)
+        
         if tbl.known_values:
             kw.update(tbl.known_values)
-        #~ logger.info("20111213 get_request_choices(%r) -> %r",self.converters,kw)
-        return self.get_choices(**kw)
+                
+        if False: # removed 20120815 #1114
+            #~ ar = tbl.request(ui,request,tbl.default_action)
+            if ar.create_kw:
+                kw.update(ar.create_kw)
+            if ar.known_values:
+                kw.update(ar.known_values)
+            if tbl.master_key:
+                kw[tbl.master_key] = ar.master_instance
+            #~ if tbl.known_values:
+                #~ kw.update(tbl.known_values)
+        #~ logger.info("20120602 get_request_choices(%r) -> %r",self.converters,kw)
+        return self.get_choices(**kw) # 20120918b
         
     def get_text_for_value(self,value,obj):
         m = getattr(self.field,'get_text_for_value',None)
@@ -168,6 +191,8 @@ class Chooser(FieldChooser):
         #~ raise NotImplementedError
         #~ assert not self.simple_values
         m = getattr(obj,"get_" + self.field.name + "_display")
+        #~ if m is None:
+            #~ raise Exception("")
         return m(value)
         #~ raise NotImplementedError("%s : Cannot get text for value %r" % (self.meth,value))
         
@@ -175,7 +200,6 @@ def check_for_chooser(model,field):
     methname = field.name + "_choices"
     m = get_class_attr(model,methname)
     if m is not None:
-        #~ n += 1
         ch = Chooser(model,field,m)
         setattr(field,'_lino_chooser',ch)
         #~ logger.debug("Installed %s",ch)
@@ -184,7 +208,7 @@ def check_for_chooser(model,field):
 
 
 def discover():
-    logger.info("Discovering choosers...")
+    logger.debug("Discovering choosers...")
     #~ logger.debug("Instantiate model reports...")
     for model in models.get_models():
         #~ n = 0
@@ -202,7 +226,15 @@ def uses_simple_values(fld):
     ch = get_for_field(fld)
     if ch is not None:
         return ch.simple_values
-    if fld.choices and type(fld.choices[0]) in (list,tuple):
+    choices = list(fld.choices)
+    #~ if choices is None:
+        #~ return True
+    #~ if callable(choices): 
+        #~ choices = choices()
+    if len(choices) == 0:
+        return True
+        #~ raise Exception("%s has no chooser but an empty `choices` attribute" % fld)
+    if type(choices[0]) in (list,tuple):
         return False
     return True
 
@@ -218,6 +250,7 @@ def chooser(**options):
             #~ print 20101220, args
             return fn(*args)
         wrapped.context_params = fn.func_code.co_varnames[1:fn.func_code.co_argcount]
+        #~ 20120918b wrapped.context_params = fn.func_code.co_varnames[2:fn.func_code.co_argcount]
         for k,v in options.items():
             setattr(wrapped,k,v)
         return classmethod(wrapped)
